package com.factory.controller.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.web.entity.excel.Utility;
import com.factory.web.entity.fa.FirstArticleTitle;
import com.factory.web.entity.fa.FirstArticleTitleBlob;
import com.factory.web.entity.pm.OnsiteTitle;
import com.factory.web.entity.pm.OnsiteTitleBlob;
import com.factory.web.entity.pm.req.SearchPmCheckRecordListReq;
import com.factory.web.service.FileService;
import com.factory.web.service.fa.FirstArticleService;
import com.factory.web.service.fa.FirstArticleTitleBlobService;
import com.factory.web.service.impl.DownLoadParam;
import com.factory.web.service.impl.FileEntry;
import com.factory.web.service.impl.FileResultInfo;
import com.factory.web.service.impl.FileServiceImpl;
import com.factory.web.service.pm.OnsiteTitleBlobService;
import com.factory.web.service.pm.PreventMistakeService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFClientAnchor;
import org.apache.poi.xssf.usermodel.XSSFDrawing;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;


@Api(tags = "本特勒 - 附件上传下载")
@RestController
@RequestMapping("/benteler/fileupload")
public class FileBlobController {

    @Autowired
    private FileService fileService;
    @Autowired
    private OnsiteTitleBlobService blobService;
    @Autowired
    private FirstArticleTitleBlobService blobService2;
    @Autowired
    private PreventMistakeService preventMistakeService;

    @Autowired
    private FirstArticleService firstArticleService;


    @PostMapping("/addFile")
    @ApiOperation(value = "上传附件-附件参数名称file")
    public ResponseBean addFile(@RequestParam("file") MultipartFile file) throws IOException {
        FileEntry fileEntry = FileEntry.builder().file(file.getBytes())
                .fileNm(file.getOriginalFilename()).build();
        fileService.save(fileEntry);
        FileResultInfo result = FileResultInfo.builder().id(fileEntry.getId()).fileNm(fileEntry.getFileNm()).build();
        return ResponseBeanBuilder.ok(result);
    }


    @GetMapping("/downloadFile")
    @ApiOperation(value = "按id下载附件测试")
    public void downFile(@RequestParam("id") long id, HttpServletResponse response) throws IOException {
        download(id, response);
    }

    @GetMapping("/downloadFileForPlan")
    @ApiOperation(value = "按计划id下载附件纠错")
    public void downloadFileForPlan(DownLoadParam param, HttpServletResponse response) throws IOException {

        long id = -1;
        if (param.getType() == 2) {
            OnsiteTitle onsiteTitle = preventMistakeService.getTitle(param.getPlanId());
            id = Long.parseLong(onsiteTitle.getUploadAttachmentId());
        } else {
            FirstArticleTitle title = firstArticleService.getFaTitle(param.getPlanId());
            id = Long.parseLong(title.getUploadAttachmentId());
        }

        download(id, response);
    }


    private void download(long id, HttpServletResponse response) {
        FileEntry file = fileService.getById(id);
        byte[] buff = null;
        if (file != null) {
            String[] suffix = file.getFileNm().split("\\.");
            String fileType = suffix[suffix.length - 1];
            if ("jpg,jepg,jpeg,gif,png".contains(fileType)) {//图片类型
                response.setContentType("image/" + fileType);
            } else if ("pdf".contains(fileType)) {//pdf类型
                response.setContentType("application/pdf");
            } else if ("xls".contains(fileType)) {//pdf类型
                response.setContentType("application/vnd.ms-excel");
            } else if ("xlsx".contains(fileType)) {//pdf类型
                response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            } else {//自动判断下载文件类型
                response.setContentType("application/octet-stream");
                String fileName = System.currentTimeMillis() + "." + suffix[suffix.length - 1];
                response.setHeader("Content-Disposition", "attachment; filename=" + fileName);
            }
            buff = file.getFile();
        } else {
            response.setContentType("text/plain");
            buff = new String("no file").getBytes(StandardCharsets.UTF_8);
        }
        OutputStream outputStream = null;
        try {
            outputStream = response.getOutputStream();
            outputStream.write(buff);
            outputStream.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }


    @GetMapping("/controlled/onsite")
    @ApiOperation(value = "纠错受控文件下载")
    public void downloadControlledOnsite(SearchPmCheckRecordListReq param, HttpServletResponse response,HttpServletRequest request) throws IOException {
        response.setContentType("application/octet-stream");
        String fileName = System.currentTimeMillis() + "." + "xlsx";
        response.setHeader("Content-Disposition", "attachment; filename=" + fileName);
       // SearchPmCheckRecordListReq pp=SearchPmCheckRecordListReq.builder().bentelerPlanId(27L).typeFlag(0).yearMonth("2021-07").build();
        Workbook wb =fileService.onsiteControlledExcel( param,request,true);
        OutputStream outputStream = null;
        try {
            outputStream = response.getOutputStream();
            wb.write(outputStream);
            outputStream.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    @GetMapping("/onsite/work/report")
    @ApiOperation(value = "纠错报表下载")
    public void downloadOnsiteReport(SearchPmCheckRecordListReq param, HttpServletResponse response,HttpServletRequest request) throws IOException {
        response.setContentType("application/octet-stream");
        String fileName = System.currentTimeMillis() + "." + "xlsx";
        response.setHeader("Content-Disposition", "attachment; filename=" + fileName);
        Workbook wb =fileService.onsiteControlledExcel( param,request,false);
        OutputStream outputStream = null;
        try {
            outputStream = response.getOutputStream();
            wb.write(outputStream);
            outputStream.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    @GetMapping("/addIniPlan")
    public void addIniPlan(HttpServletRequest request) throws IOException {

        OnsiteTitle title= OnsiteTitle.builder().department("1111").issueDate("2020-01-01").issureLevel("hjgsajhasd")
                .productionLine("12r3y12t3").replace("jsadhgas").uploadAttachmentId("24").build();

        ByteArrayOutputStream baos =new ByteArrayOutputStream();
        ObjectOutputStream oos =new ObjectOutputStream(baos);
        oos.writeObject(title);
        OnsiteTitleBlob testEntry = OnsiteTitleBlob.builder().bentelerPlanId(Long.parseLong("27"))//设置机会id
                .titleContent(baos.toByteArray()).build();
        blobService.save(testEntry);

        FirstArticleTitle ftitle= FirstArticleTitle.builder().project("1").partname("1").partNoOem("2").productgroup("1")
                .gpin("asdasd").language("asd").oem("asdas").statusDoc("asdasd").version("asdas")
                .batMaterial("asdasd").lastChange("asdasd").dmbA("asbdjasd").uploadAttachmentId("24").build();
        baos =new ByteArrayOutputStream();
        oos =new ObjectOutputStream(baos);
        oos.writeObject(ftitle);
        FirstArticleTitleBlob testEntry2 = FirstArticleTitleBlob.builder().bentelerPlanId(Long.parseLong("38"))//设置机会id
                .titleContent(baos.toByteArray()).build();
        blobService2.save(testEntry2);
        System.out.println("11111111111111111111111111111111111111111");
    }

//    @GetMapping("/getIniPlan")
//    public void getIniPlan(HttpServletRequest request) {
//        Long versionId = Long.valueOf(2);
//        FileEntry pp= testService.getById(versionId);
//        pp.setTitleContentStr(new String(pp.getTitleContent(),StandardCharsets.UTF_8));
//        System.out.println(pp.getTitleContentStr());
//    }


}
