package com.factory.web.service.impl.fa;

import cn.hutool.core.convert.Convert;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.enumPo.PlanType;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.config.MyHandler;
import com.factory.user.entity.vo.res.FileRes;
import com.factory.user.service.CommonService;
import com.factory.web.entity.fa.*;
import com.factory.web.entity.fa.req.*;
import com.factory.web.entity.fa.res.*;
import com.factory.web.entity.pm.OnsiteApprovalPlan;
import com.factory.web.entity.pm.res.ApprovalUserRes;
import com.factory.web.mapper.fa.FirstArticleMapper;
import com.factory.web.service.BentelerPlanService;
import com.factory.web.service.fa.*;
import com.factory.web.service.pm.OnsiteApprovalPlanService;
import com.factory.web.service.pm.PreventMistakeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.*;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;


/**
 * 首件计划
 *
 * @author wzy
 * @date 2021-07-09 12:00:00
 */
@Service("firstArticleService")
@Transactional(rollbackFor = Exception.class)
public class FirstArticleServiceImpl extends ServiceImpl<FirstArticleMapper, BentelerPlan> implements FirstArticleService {

    private static long ROLE_ID = -1;

    @Autowired
    private FirstArticleMapper firstArticleMapper;

    @Autowired
    private PreventMistakeService preventMistakeService;

    @Autowired
    private OnsiteApprovalPlanService onsiteApprovalPlanService;

    @Autowired
    private BentelerPlanService planService;

    @Autowired
    private FirstArticleCalendarService firstArticleCalendarService;

    @Autowired
    private FirstArticleTitleBlobService firstArticleTitleBlobService;

    @Autowired
    private FirstArticleWorkshopService firstArticleWorkshopService;

    @Autowired
    private FirstArticleItemService firstArticleItemService;

    @Autowired
    private FirstArticleItemResultService firstArticleItemResultService;

    @Autowired
    private FirstArticleTaskService firstArticleTaskService;

    @Autowired
    private FirstArticleApprovalTaskService firstArticleApprovalTaskService;

    @Autowired
    private FirstArticleTaskHisBlobService firstArticleTaskHisBlobService;

    @Autowired
    private CommonService commonService;


    /**
     * 新建防错计划
     *
     * @param req
     * @return
     */
    @Override
    public ResponseBean addFaPlan(AddFaPlanReq req) {
        /* 校验计划名称是否存在 */
        if (this.checkPlanNameExist(req.getBentelerPlanName())) {
            return ResponseBeanBuilder.fail("计划名称已存在");
        }

        /* 添加防错计划 */
        BentelerPlan plan = BentelerPlan.builder()
                .planType(PlanType.首件.value)
                .name(req.getBentelerPlanName())
                .pauseFlag(false)
                .effectiveDate(req.getEffectiveDate())
                .build();
        planService.save(plan);

        /* 添加首件、200件点检规则 */
        List<FirstArticleCalendarReq> firstArticleCalendarList = req.getFirstArticleCalendarList();
        for (FirstArticleCalendarReq firstArticleCalendarReq : firstArticleCalendarList) {
            FirstArticleCalendar firstArticleCalendar = FirstArticleCalendar.builder()
                    .bentelerPlanId(plan.getId())
                    .startDay(firstArticleCalendarReq.getStartDay())
                    .endDay(firstArticleCalendarReq.getEndDay())
                    .dayValidate(firstArticleCalendarReq.getDayValidate())
                    .dayOuttime(firstArticleCalendarReq.getDayOuttime())
                    .twoHundredSubmitTime(firstArticleCalendarReq.getTwoHundredSubmitTime())
                    .userId(firstArticleCalendarReq.getUserId())
                    .typeFlag(firstArticleCalendarReq.getTypeFlag())
                    .shift(firstArticleCalendarReq.getShift())
                    .build();
            firstArticleCalendarService.save(firstArticleCalendar);
        }

        /* 默认插入一条中断点检规则 */
        FirstArticleCalendar firstArticleCalendar = FirstArticleCalendar.builder()
                .bentelerPlanId(plan.getId())
                .userId(-1L)
                .typeFlag(2) // 2中断
                .build();
        firstArticleCalendarService.save(firstArticleCalendar);

        /* 添加表头信息 */
        try {
            FirstArticleTitleBlob firstArticleTitleBlob = FirstArticleTitleBlob.builder()
                    .bentelerPlanId(plan.getId())
                    .titleContent(this.objectToByteArray(req.getFirstArticleTitle()))
                    .build();
            firstArticleTitleBlobService.save(firstArticleTitleBlob);
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        /* 添加工作站信息 */
        List<FirstArticleWorkshopReq> firstArticleWorkshopList = req.getFirstArticleWorkshopList();
        for (FirstArticleWorkshopReq firstArticleWorkshopReq : firstArticleWorkshopList) {
//			if (this.checkWsNameExist(firstArticleWorkshopReq.getWsName())) {
//				return ResponseBeanBuilder.fail("工作站名称" + firstArticleWorkshopReq.getWsName() + "已存在");
//			}

            FirstArticleWorkshop firstArticleWorkshop = FirstArticleWorkshop.builder()
                    .bentelerPlanId(plan.getId())
                    .wsName(firstArticleWorkshopReq.getWsName())
                    .plName(firstArticleWorkshopReq.getPlName())
                    .build();
            firstArticleWorkshopService.save(firstArticleWorkshop);

            /* 添加巡检项目信息 */
            List<FirstArticleItemReq> firstArticleItemList = firstArticleWorkshopReq.getFirstArticleItemList();
            for (FirstArticleItemReq firstArticleItemReq : firstArticleItemList) {
                FirstArticleItem firstArticleItem = FirstArticleItem.builder()
                        .workshopId(firstArticleWorkshop.getId())
                        .itemContent(firstArticleItemReq.getItemContent())
                        .itemStandard(firstArticleItemReq.getItemStandard())
                        .prefix(firstArticleItemReq.getPrefix())
                        .snoStart(firstArticleItemReq.getSnoStart())
                        .snoEnd(firstArticleItemReq.getSnoEnd())
                        .valFlag(firstArticleItemReq.getValFlag())
                        .valUp(firstArticleItemReq.getValUp())
                        .valDown(firstArticleItemReq.getValDown())
                        .valOk(firstArticleItemReq.getValOk())
                        .valDesc(firstArticleItemReq.getValDesc())
                        .levelInfo(firstArticleItemReq.getLevelInfo())
                        .productInfo(firstArticleItemReq.getProductInfo())
                        .toolId(firstArticleItemReq.getToolId())
                        .toolDesc(firstArticleItemReq.getToolDesc())
                        .firstFlag(firstArticleItemReq.getFirstFlag())
                        .twoHundredFlag(firstArticleItemReq.getTwoHundredFlag())
                        .catchFlag(firstArticleItemReq.getCatchFlag())
                        .build();
                firstArticleItemService.save(firstArticleItem);
            }
        }

        /* 取得审批人 approvalLevel=1级审批 type=1计划审批 */
        ApprovalUserRes approvalUserRes = preventMistakeService.getApprovalUserInfo(1, 1);
        if (approvalUserRes != null) {
            /* 添加计划一级审批 */
            OnsiteApprovalPlan onsiteApprovalPlan = OnsiteApprovalPlan.builder()
                    .bentelerPlanId(plan.getId())
                    .approvalLevel(1) // 1级审批
                    .approvalUserId(approvalUserRes.getUserId())
                    .status(0) // 0未审批
                    .build();
            onsiteApprovalPlanService.save(onsiteApprovalPlan);
        }

        return ResponseBeanBuilder.ok();
    }

    /**
     * 校验计划名是否存在
     *
     * @param name
     * @return true存在，false不存在
     */
    @Override
    public boolean checkPlanNameExist(String name) {
        int count = planService.count(
                Wrappers.<BentelerPlan>lambdaQuery().eq(BentelerPlan::getName, name));
        if (count > 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 校验工作站名是否存在
     *
     * @param wsName
     * @return true存在，false不存在
     */
    @Override
    public boolean checkWsNameExist(String wsName) {
        int count = firstArticleWorkshopService.count(
                Wrappers.<FirstArticleWorkshop>lambdaQuery().eq(FirstArticleWorkshop::getWsName, wsName));
        if (count > 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 取得防错计划信息
     *
     * @param req
     * @return 防错计划信息
     */
    @Override
    public FaPlanRes getFaPlan(SearchFaPlanReq req) {
        BentelerPlan plan = planService.getById(req.getBentelerPlanId());
        FaPlanRes faPlanRes = new FaPlanRes();
        faPlanRes.setBentelerPlanId(plan.getId());
        faPlanRes.setBentelerPlanName(plan.getName());
        faPlanRes.setPlanType(plan.getPlanType());
        faPlanRes.setEffectiveDate(plan.getEffectiveDate());
        ;

        List<FirstArticleCalendar> firstArticleCalendarList = firstArticleCalendarService.list(Wrappers.<FirstArticleCalendar>lambdaQuery()
                .eq(FirstArticleCalendar::getBentelerPlanId, plan.getId()));
        faPlanRes.setFirstArticleCalendarList(firstArticleCalendarList);

        try {
            FirstArticleTitleBlob firstArticleTitleBlob = firstArticleTitleBlobService.getOne(new LambdaQueryWrapper<FirstArticleTitleBlob>()
                    .eq(FirstArticleTitleBlob::getBentelerPlanId, req.getBentelerPlanId()));
            faPlanRes.setFirstArticleTitle((FirstArticleTitle) this.byteArrayToObject(firstArticleTitleBlob.getTitleContent()));
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        List<FirstArticleWorkshop> firstArticleWorkshopList = firstArticleWorkshopService.list(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                .eq(FirstArticleWorkshop::getBentelerPlanId, plan.getId()));
        for (FirstArticleWorkshop firstArticleWorkshop : firstArticleWorkshopList) {
            List<FirstArticleItem> firstArticleItemList = firstArticleItemService.list(Wrappers.<FirstArticleItem>lambdaQuery()
                    .eq(FirstArticleItem::getWorkshopId, firstArticleWorkshop.getId()));
            firstArticleWorkshop.setFirstArticleItemList(firstArticleItemList);
        }
        faPlanRes.setFirstArticleWorkshopList(firstArticleWorkshopList);

        return faPlanRes;
    }


    /**
     * 修改首件计划
     *
     * @param req
     * @return
     */
    @Override
    public ResponseBean editFaPlan(EditFaPlanReq req) {

        BentelerPlan oldPlan = planService.getById(req.getBentelerPlanId());
        if (!req.getBentelerPlanName().equals(oldPlan.getName())) {
            /* 校验计划名称是否存在 */
            if (this.checkPlanNameExist(req.getBentelerPlanName())) {
                return ResponseBeanBuilder.fail("计划名称已存在");
            }
        }

        /* 修改防错计划 */
        BentelerPlan plan = BentelerPlan.builder()
                .id(req.getBentelerPlanId())
                .name(req.getBentelerPlanName())
                .effectiveDate(req.getEffectiveDate())
                .build();
        planService.updateById(plan);

        /* 修改首件、200件点检规则 */
        /* 删除首件、200件点检规则 */
        firstArticleCalendarService.remove(Wrappers.<FirstArticleCalendar>lambdaQuery()
                .eq(FirstArticleCalendar::getBentelerPlanId, plan.getId()));
        /* 添加首件、200件点检规则 */
        List<FirstArticleCalendarReq> firstArticleCalendarList = req.getFirstArticleCalendarList();
        for (FirstArticleCalendarReq firstArticleCalendarReq : firstArticleCalendarList) {
            FirstArticleCalendar firstArticleCalendar = FirstArticleCalendar.builder()
                    .bentelerPlanId(plan.getId())
                    .startDay(firstArticleCalendarReq.getStartDay())
                    .endDay(firstArticleCalendarReq.getEndDay())
                    .dayValidate(firstArticleCalendarReq.getDayValidate())
                    .dayOuttime(firstArticleCalendarReq.getDayOuttime())
                    .twoHundredSubmitTime(firstArticleCalendarReq.getTwoHundredSubmitTime())
                    .userId(firstArticleCalendarReq.getUserId())
                    .typeFlag(firstArticleCalendarReq.getTypeFlag())
                    .shift(firstArticleCalendarReq.getShift())
                    .build();
            firstArticleCalendarService.save(firstArticleCalendar);
        }
        /* 默认插入一条中断点检规则 */
        FirstArticleCalendar firstArticleCalendar = FirstArticleCalendar.builder()
                .bentelerPlanId(plan.getId())
                .userId(-1L)
                .typeFlag(2) // 2中断
                .build();
        firstArticleCalendarService.save(firstArticleCalendar);

        /* 修改表头信息 */
        try {
            LambdaUpdateWrapper<FirstArticleTitleBlob> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
            lambdaUpdateWrapper.eq(FirstArticleTitleBlob::getBentelerPlanId, plan.getId())
                    .set(FirstArticleTitleBlob::getTitleContent, this.objectToByteArray(req.getFirstArticleTitle()));
            firstArticleTitleBlobService.update(lambdaUpdateWrapper);
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        /* 修改工作站、巡检项目 */
        /* 删除巡检项目 */
        List<FirstArticleWorkshop> firstArticleWorkshopListTemp = firstArticleWorkshopService.list(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                .eq(FirstArticleWorkshop::getBentelerPlanId, plan.getId()));
        List<Long> workshopIdList = new ArrayList<Long>();
        for (FirstArticleWorkshop firstArticleWorkshop : firstArticleWorkshopListTemp) {
            workshopIdList.add(firstArticleWorkshop.getId());
        }
        firstArticleItemService.remove(Wrappers.<FirstArticleItem>lambdaQuery()
                .in(FirstArticleItem::getWorkshopId, workshopIdList));
        /* 删除工作站 */
        firstArticleWorkshopService.remove(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                .eq(FirstArticleWorkshop::getBentelerPlanId, plan.getId()));
        /* 添加工作站信息 */
        List<FirstArticleWorkshopReq> firstArticleWorkshopList = req.getFirstArticleWorkshopList();
        for (FirstArticleWorkshopReq firstArticleWorkshopReq : firstArticleWorkshopList) {
//			if (this.checkWsNameExist(firstArticleWorkshopReq.getWsName())) {
//				return ResponseBeanBuilder.fail("工作站名称" + firstArticleWorkshopReq.getWsName() + "已存在");
//			}

            FirstArticleWorkshop firstArticleWorkshop = FirstArticleWorkshop.builder()
                    .bentelerPlanId(plan.getId())
                    .wsName(firstArticleWorkshopReq.getWsName())
                    .plName(firstArticleWorkshopReq.getPlName())
                    .build();
            firstArticleWorkshopService.save(firstArticleWorkshop);

            /* 添加巡检项目信息 */
            List<FirstArticleItemReq> firstArticleItemList = firstArticleWorkshopReq.getFirstArticleItemList();
            for (FirstArticleItemReq firstArticleItemReq : firstArticleItemList) {
                FirstArticleItem firstArticleItem = FirstArticleItem.builder()
                        .workshopId(firstArticleWorkshop.getId())
                        .itemContent(firstArticleItemReq.getItemContent())
                        .itemStandard(firstArticleItemReq.getItemStandard())
                        .prefix(firstArticleItemReq.getPrefix())
                        .snoStart(firstArticleItemReq.getSnoStart())
                        .snoEnd(firstArticleItemReq.getSnoEnd())
                        .valFlag(firstArticleItemReq.getValFlag())
                        .valUp(firstArticleItemReq.getValUp())
                        .valDown(firstArticleItemReq.getValDown())
                        .valOk(firstArticleItemReq.getValOk())
                        .valDesc(firstArticleItemReq.getValDesc())
                        .levelInfo(firstArticleItemReq.getLevelInfo())
                        .productInfo(firstArticleItemReq.getProductInfo())
                        .toolId(firstArticleItemReq.getToolId())
                        .toolDesc(firstArticleItemReq.getToolDesc())
                        .firstFlag(firstArticleItemReq.getFirstFlag())
                        .twoHundredFlag(firstArticleItemReq.getTwoHundredFlag())
                        .catchFlag(firstArticleItemReq.getCatchFlag())
                        .build();
                firstArticleItemService.save(firstArticleItem);
            }
        }

        /* 删除之前的审批记录 */
        onsiteApprovalPlanService.remove(Wrappers.<OnsiteApprovalPlan>lambdaQuery()
                .eq(OnsiteApprovalPlan::getBentelerPlanId, plan.getId()));
        /* 取得审批人 approvalLevel=1级审批 type=1计划审批 */
        ApprovalUserRes approvalUserRes = preventMistakeService.getApprovalUserInfo(1, 1);
        if (approvalUserRes != null) {
            /* 添加计划一级审批 */
            OnsiteApprovalPlan onsiteApprovalPlan = OnsiteApprovalPlan.builder()
                    .bentelerPlanId(plan.getId())
                    .approvalLevel(1) // 1级审批
                    .approvalUserId(approvalUserRes.getUserId())
                    .status(0) // 0未审批
                    .build();
            onsiteApprovalPlanService.save(onsiteApprovalPlan);
        }

        return ResponseBeanBuilder.ok();
    }

    /**
     * 取得首件计划点检列表
     *
     * @param req
     * @return 首件计划点检列表
     */
    @Override
    public ResponseBean<PageBean<FaPlanListRes>> getFaPlanCheckList(SearchFaPlanCheckListReq req) {
        Page<FaPlanListRes> page = new Page<>(req.getPage(), req.getSize());
        Page<FaPlanListRes> listResPage = firstArticleMapper.getFaPlanCheckList(page, req);
        return ResponseBeanBuilder.ok(listResPage, Function.identity());
    }


    /**
     * 取得Pad端防错计划列表
     *
     * @param req
     * @return Pad端防错计划列表
     */
    @Override
    public ResponseBean<List<FaPlanListForPadRes>> getFaPlanListForPad(SearchFaPlanListForPadReq req) {
        List<FaPlanListForPadRes> listRes = firstArticleMapper.getFaPlanListForPad(req);
        return ResponseBeanBuilder.ok(listRes, Function.identity());
    }

    /**
     * 得Pad端首件计划对应的工作站
     *
     * @param req
     * @return 工作站
     */
    @Override
    public ResponseBean<List<FirstArticleWorkshop>> getFaWorkshopListForPad(SearchFaWorkshopListForPadReq req) {
        /* 取得task信息 */
        SearchFaTaskInfoReq searchFaTaskInfoReq = new SearchFaTaskInfoReq();
        searchFaTaskInfoReq.setBentelerPlanId(req.getBentelerPlanId());
        searchFaTaskInfoReq.setTypeFlag(req.getTypeFlag());

        FirstArticleTask faTaskInfo = null;
        if (req.getTypeFlag() == 0 || req.getTypeFlag() == 1) { // 0表示首件 1表示200件
            searchFaTaskInfoReq.setUserId(req.getUserId());
            faTaskInfo = this.getFaTaskInfo(searchFaTaskInfoReq);
        } else { // 2表示中断
            /* 取得中断最新的task */
            faTaskInfo = this.getFaTaskInfoCatchLatest(searchFaTaskInfoReq);
        }

        long firstArticleTaskId = 0;
        if (faTaskInfo != null) {
            firstArticleTaskId = faTaskInfo.getId();
        }

        List<FirstArticleWorkshop> listRes = firstArticleMapper.getFaWorkshopListForPad(req);

        SearchFaItemListForPadReq searchFaItemListReq = null;
        SearchFaWsItemResultCountReq searchFaWsItemResultCountReq = null;
        for (FirstArticleWorkshop firstArticleWorkshop : listRes) {
            /* 取得点检项目 */
            searchFaItemListReq = new SearchFaItemListForPadReq();
            searchFaItemListReq.setWorkshopId(firstArticleWorkshop.getId());
            searchFaItemListReq.setTypeFlag(req.getTypeFlag());
            List<FaItemListForPadRes> listResTemp = firstArticleMapper.getFaItemListForPad(searchFaItemListReq);
            /* 计算点检项目数 */
            int itemCount = 0;
            for (FaItemListForPadRes faItem : listResTemp) {
                int start = faItem.getSnoStart();
                int end = faItem.getSnoEnd();

                itemCount += (end - start + 1);
            }

            searchFaWsItemResultCountReq = new SearchFaWsItemResultCountReq();
            searchFaWsItemResultCountReq.setFirstArticleTaskId(firstArticleTaskId);
            searchFaWsItemResultCountReq.setWorkshopId(firstArticleWorkshop.getId());
//    		int itemCount = firstArticleMapper.getFaWsItemCount(searchFaWsItemResultCountReq);
            int itemResultCount = firstArticleMapper.getFaWsItemResultCount(searchFaWsItemResultCountReq);
            if (itemCount == itemResultCount) {
                firstArticleWorkshop.setAllCheckFlag(1);
            } else {
                firstArticleWorkshop.setAllCheckFlag(0);
            }
        }

        return ResponseBeanBuilder.ok(listRes, Function.identity());
    }

    /**
     * 取得Pad端巡检项目
     *
     * @param req
     * @return 巡检项目
     */
    @Override
    public ResponseBean<List<FaItemListForPadRes>> getFaItemListForPad(SearchFaItemListForPadReq req) {
        /* 取得task信息 */
        SearchFaTaskInfoReq searchFaTaskInfoReq = new SearchFaTaskInfoReq();
        searchFaTaskInfoReq.setBentelerPlanId(req.getBentelerPlanId());
        searchFaTaskInfoReq.setTypeFlag(req.getTypeFlag());

        FirstArticleTask faTaskInfo = null;
        if (req.getTypeFlag() == 0 || req.getTypeFlag() == 1) { // 0表示首件 1表示200件
            searchFaTaskInfoReq.setUserId(req.getUserId());
            faTaskInfo = this.getFaTaskInfo(searchFaTaskInfoReq);
        } else { // 2表示中断
            /* 取得中断最新的task */
            faTaskInfo = this.getFaTaskInfoCatchLatest(searchFaTaskInfoReq);
        }

        long firstArticleTaskId = 0;
        if (faTaskInfo != null) {
            firstArticleTaskId = faTaskInfo.getId();
        }

        /* 取得点检项目 */
        List<FaItemListForPadRes> listResTemp = firstArticleMapper.getFaItemListForPad(req);

        List<FaItemListForPadRes> listRes = new ArrayList<FaItemListForPadRes>();
        for (FaItemListForPadRes faItemListForPadRes : listResTemp) {
            /* 序号拆分（从1到10，就拆分成10条） */
            int snoStart = faItemListForPadRes.getSnoStart();
            int snoEnd = faItemListForPadRes.getSnoEnd();

            FaItemListForPadRes itemInfo = null;
            for (int i = snoStart; i <= snoEnd; i++) {
                itemInfo = new FaItemListForPadRes();

                itemInfo.setSno(i);
                itemInfo.setId(faItemListForPadRes.getId());
                itemInfo.setWorkshopId(faItemListForPadRes.getWorkshopId());
                itemInfo.setItemContent(faItemListForPadRes.getItemContent());
                itemInfo.setItemStandard(faItemListForPadRes.getItemStandard());
                itemInfo.setPrefix(faItemListForPadRes.getPrefix());
                itemInfo.setSnoStart(faItemListForPadRes.getSnoStart());
                itemInfo.setSnoEnd(faItemListForPadRes.getSnoEnd());
                itemInfo.setValFlag(faItemListForPadRes.getValFlag());
                itemInfo.setValUp(faItemListForPadRes.getValUp());
                itemInfo.setValDown(faItemListForPadRes.getValDown());
                itemInfo.setValOk(faItemListForPadRes.getValOk());
                itemInfo.setValDesc(faItemListForPadRes.getValDesc());
                itemInfo.setLevelInfo(faItemListForPadRes.getLevelInfo());
                itemInfo.setProductInfo(faItemListForPadRes.getProductInfo());
                itemInfo.setToolId(faItemListForPadRes.getToolId());
                itemInfo.setToolDesc(faItemListForPadRes.getToolDesc());
                itemInfo.setFirstFlag(faItemListForPadRes.getFirstFlag());
                itemInfo.setTwoHundredFlag(faItemListForPadRes.getTwoHundredFlag());
                itemInfo.setCatchFlag(faItemListForPadRes.getCatchFlag());
                itemInfo.setValUnit(faItemListForPadRes.getValUnit());

                /* 取得检查结果 */
                FirstArticleItemResult firstArticleItemResult = firstArticleItemResultService.getOne(Wrappers.<FirstArticleItemResult>lambdaQuery()
                        .eq(FirstArticleItemResult::getFirstArticleTaskId, firstArticleTaskId)
                        .eq(FirstArticleItemResult::getFirstArticleItemId, itemInfo.getId())
                        .eq(FirstArticleItemResult::getSno, itemInfo.getSno()));
                if (firstArticleItemResult != null) {
                    itemInfo.setResult(firstArticleItemResult.getResult());
                    itemInfo.setValCheck(firstArticleItemResult.getVal());
                    itemInfo.setValDescCheck(firstArticleItemResult.getValDesc());
                    itemInfo.setToolDescCheck(firstArticleItemResult.getToolDesc());
                }

                listRes.add(itemInfo);
            }
        }

        return ResponseBeanBuilder.ok(listRes, Function.identity());
    }

    /**
     * 添加巡检项目检查结果
     *
     * @param req
     * @return
     */
    @Override
    public ResponseBean addFaItemResult(AddFaItemResultListReq req) {
        /* 取得task信息 */
        SearchFaTaskInfoReq searchFaTaskInfoReq = new SearchFaTaskInfoReq();
        searchFaTaskInfoReq.setBentelerPlanId(req.getBentelerPlanId());
        searchFaTaskInfoReq.setTypeFlag(req.getTypeFlag());
        if (req.getTypeFlag() == 0 || req.getTypeFlag() == 1) { // 0表示首件 1表示200件
            searchFaTaskInfoReq.setUserId(req.getUserId());
        } else { // 2表示中断
            searchFaTaskInfoReq.setStatus(0); // task状态为0初始化
        }
        FirstArticleTask faTaskInfo = this.getFaTaskInfo(searchFaTaskInfoReq);

        long firstArticleTaskId = 0;
        if (req.getTypeFlag() == 0 || req.getTypeFlag() == 1) { // 0表示首件 1表示200件
            if (faTaskInfo == null) {
                return ResponseBeanBuilder.fail("计划任务不存在！");
            } else {
                if (faTaskInfo.getStatus() == 0) {
                    firstArticleTaskId = faTaskInfo.getId();
                } else {
                    return ResponseBeanBuilder.fail("该计划任务已处理！无需重复提交！");
                }
            }
        } else { // 2表示中断
            if (faTaskInfo == null) { // 没有初始化状态的task，新建一个
                /* 取得中断对应的规则 */
                FirstArticleCalendar firstArticleCalendar = firstArticleCalendarService.getOne(new LambdaQueryWrapper<FirstArticleCalendar>()
                        .eq(FirstArticleCalendar::getBentelerPlanId, req.getBentelerPlanId())
                        .eq(FirstArticleCalendar::getTypeFlag, 2));

                /* 创建一个新的中断task */
                FirstArticleTask firstArticleTask = FirstArticleTask.builder()
                        .firstArticleCalendarId(firstArticleCalendar.getId())
                        .status(0)
                        .taskTime(LocalDate.now())
                        .remindTaskFlag(0)
                        .remindNoworkFlag(0)
                        .build();
                firstArticleTaskService.save(firstArticleTask);

                firstArticleTaskId = firstArticleTask.getId();
            } else { // 存在一个初始化状态的task，就用这个
                firstArticleTaskId = faTaskInfo.getId();
            }
        }

        List<AddFaItemResultReq> addFaItemResultList = req.getAddFaItemResultList();
        /* 删除上一次保存结果 */
        List<Long> firstArticleItemIdList = new ArrayList<Long>();
        for (AddFaItemResultReq itemResult : addFaItemResultList) {
            firstArticleItemIdList.add(itemResult.getFirstArticleItemId());
        }

        firstArticleItemResultService.remove(Wrappers.<FirstArticleItemResult>lambdaQuery()
                .eq(FirstArticleItemResult::getFirstArticleTaskId, firstArticleTaskId)
                .in(FirstArticleItemResult::getFirstArticleItemId, firstArticleItemIdList));

        /* 添加新的检查结果 */
//    	List<AddFaItemResultReq> addFaItemResultList = req.getAddFaItemResultList();
        for (AddFaItemResultReq addFaItemResult : addFaItemResultList) {
            FirstArticleItemResult firstArticleItemResult = FirstArticleItemResult.builder()
                    .firstArticleTaskId(firstArticleTaskId)
                    .firstArticleItemId(addFaItemResult.getFirstArticleItemId())
                    .val(addFaItemResult.getVal())
                    .valDesc(addFaItemResult.getValDesc())
                    .result(addFaItemResult.getResult())
                    .prefix(addFaItemResult.getPrefix())
                    .sno(addFaItemResult.getSno())
                    .toolDesc(addFaItemResult.getToolDesc())
                    .build();
            firstArticleItemResultService.save(firstArticleItemResult);
        }

        return ResponseBeanBuilder.ok();
    }

    /**
     * 任务提交
     *
     * @param req
     * @return
     */
    @Override
    public ResponseBean submitFaTask(SubmitFaTaskReq req) {
        /* 取得task信息 */
        SearchFaTaskInfoReq searchFaTaskInfoReq = new SearchFaTaskInfoReq();
        searchFaTaskInfoReq.setBentelerPlanId(req.getBentelerPlanId());
        searchFaTaskInfoReq.setTypeFlag(req.getTypeFlag());
        if (req.getTypeFlag() == 0 || req.getTypeFlag() == 1) { // 0表示首件 1表示200件
            searchFaTaskInfoReq.setUserId(req.getUserId());
        } else { // 2表示中断
            searchFaTaskInfoReq.setStatus(0); // task状态为0初始化
        }
        FirstArticleTask faTaskInfo = this.getFaTaskInfo(searchFaTaskInfoReq);

        long firstArticleTaskId = 0;
        if (req.getTypeFlag() == 0 || req.getTypeFlag() == 1) { // 0表示首件 1表示200件
            if (faTaskInfo == null) {
                return ResponseBeanBuilder.fail("计划任务不存在！");
            } else {
                if (faTaskInfo.getStatus() == 0 || faTaskInfo.getStatus() == 3) { // 首次提交和重检提交
                    firstArticleTaskId = faTaskInfo.getId();
                } else {
                    return ResponseBeanBuilder.fail("该计划任务已处理！无需重复提交！");
                }
            }
        } else { // 2表示中断
            if (faTaskInfo == null) { // 没有初始化状态的task，新建一个
                return ResponseBeanBuilder.fail("请先完成点检！");
            } else { // 存在一个初始化状态的task，就用这个
                firstArticleTaskId = faTaskInfo.getId();
            }
        }

        /* 取得task下的所有条目检查结果 */
        List<FirstArticleItemResult> firstArticleItemResultList = firstArticleItemResultService.list(Wrappers.<FirstArticleItemResult>lambdaQuery()
                .eq(FirstArticleItemResult::getFirstArticleTaskId, firstArticleTaskId));

        boolean flag = true; // true表示巡检项目都通过，false表示巡检项目有不通过的
        for (FirstArticleItemResult firstArticleItemResult : firstArticleItemResultList) {
            if (firstArticleItemResult.getResult() == 1) {
                flag = false;
                break;
            }
        }

        if (flag) { // 巡检项目都通过
            FirstArticleTask firstArticleTask = FirstArticleTask.builder()
                    .id(firstArticleTaskId)
                    .status(4)
                    .build();
            firstArticleTaskService.updateById(firstArticleTask);
        } else { // 巡检项目有不通过的
            /* 更新任务为1待审核状态。等待后续审核 */
            FirstArticleTask firstArticleTask = FirstArticleTask.builder()
                    .id(firstArticleTaskId)
                    .status(1)
                    .build();
            firstArticleTaskService.updateById(firstArticleTask);

            /* 删除之前的审批记录 */
            firstArticleApprovalTaskService.remove(Wrappers.<FirstArticleApprovalTask>lambdaQuery()
                    .eq(FirstArticleApprovalTask::getFirstArticleTaskId, firstArticleTask.getId()));

            /* 取得审批人 approvalLevel=1级审批 type=2首件任务审批 */
            ApprovalUserRes approvalUserRes = preventMistakeService.getApprovalUserInfo(1, 2);
            if (approvalUserRes != null) {
                /* 添加任务一级审批 */
                FirstArticleApprovalTask firstArticleApprovalTask = FirstArticleApprovalTask.builder()
                        .firstArticleTaskId(firstArticleTask.getId())
                        .approvalLevel(1) // 1级审批
                        .approvalUserId(approvalUserRes.getUserId())
                        .status(0) // 0初始化
                        .build();
                firstArticleApprovalTaskService.save(firstArticleApprovalTask);
            }
        }

        /* 更新中断的操作人 */
        if (req.getTypeFlag() == 2) {
            LambdaUpdateWrapper<FirstArticleCalendar> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
            lambdaUpdateWrapper.eq(FirstArticleCalendar::getBentelerPlanId, req.getBentelerPlanId())
                    .eq(FirstArticleCalendar::getTypeFlag, 2)
                    .set(FirstArticleCalendar::getUserId, req.getUserId());
            firstArticleCalendarService.update(lambdaUpdateWrapper);
        }

        return ResponseBeanBuilder.ok();
    }

    /**
     * 取得不放行任务对应的工作站
     *
     * @param req
     * @return 不放行任务对应的工作站
     */
    @Override
    public ResponseBean<List<FaWorkshopNoGoListRes>> getFaWorkshopNoGoList(SearchFaWorkshopNoGoListReq req) {
        List<FaWorkshopNoGoListRes> faWorkshopNoGoList = firstArticleMapper.getFaWorkshopNoGoList(req);
        return ResponseBeanBuilder.ok(faWorkshopNoGoList, Function.identity());
    }

    /**
     * 取得不放行任务对应的点检项目
     *
     * @param req
     * @return 不放行任务对应的点检项目
     */
    @Override
    public ResponseBean<List<FaItemNoGoListRes>> getFaItemNoGoList(SearchFaItemNoGoListReq req) {
        List<FaItemNoGoListRes> faItemNoGoList = firstArticleMapper.getFaItemNoGoList(req);
        return ResponseBeanBuilder.ok(faItemNoGoList, Function.identity());
    }


    /**
     * 修改巡检项目检查结果(这里只有重检走这里)
     *
     * @param req
     * @return
     */
    @Override
    public ResponseBean editFaItemResult(EditFaItemResultListReq req) {
        /* 更新巡检项目检查结果 */
        List<EditFaItemResultReq> editFaItemResultList = req.getEditFaItemResultList();
        for (EditFaItemResultReq editFaItemResult : editFaItemResultList) {
            FirstArticleItemResult firstArticleItemResult = FirstArticleItemResult.builder()
                    .id(editFaItemResult.getId())
                    .val(editFaItemResult.getVal())
                    .valDesc(editFaItemResult.getValDesc())
                    .result(editFaItemResult.getResult())
                    .prefix(editFaItemResult.getPrefix())
                    .sno(editFaItemResult.getSno())
                    .toolDesc(editFaItemResult.getToolDesc())
                    .build();
            firstArticleItemResultService.updateById(firstArticleItemResult);
        }

        return ResponseBeanBuilder.ok();
    }


    /**
     * 任务和巡检结果记录历史
     *
     * @param onsiteTaskId
     * @return
     */
    @Override
    public void setFirstArticleTaskHis(Long firstArticleTaskId) {
        /* 任务和巡检结果记录历史 */
        FaTaskRes faTaskRes = new FaTaskRes();
        FirstArticleTask firstArticleTask = firstArticleTaskService.getById(firstArticleTaskId);
        faTaskRes.setFirstArticleTaskId(firstArticleTask.getId());
        faTaskRes.setFirstArticleCalendarId(firstArticleTask.getFirstArticleCalendarId());
        faTaskRes.setStatus(firstArticleTask.getStatus());
        faTaskRes.setTaskTime(firstArticleTask.getTaskTime());
        faTaskRes.setRemindTaskFlag(firstArticleTask.getRemindTaskFlag());
        faTaskRes.setRemindNoworkFlag(firstArticleTask.getRemindNoworkFlag());

        List<FirstArticleItemResult> firstArticleItemResultList = firstArticleItemResultService.list(Wrappers.<FirstArticleItemResult>lambdaQuery()
                .eq(FirstArticleItemResult::getFirstArticleTaskId, firstArticleTaskId));
        faTaskRes.setFirstArticleItemResultList(firstArticleItemResultList);

        try {
            FirstArticleTaskHisBlob firstArticleTaskHisBlob = FirstArticleTaskHisBlob.builder()
                    .firstArticleTaskId(firstArticleTaskId)
                    .hisContent(this.objectToByteArray(faTaskRes))
                    .build();
            firstArticleTaskHisBlobService.save(firstArticleTaskHisBlob);
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * 取得表头信息
     *
     * @param bentelerPlanId
     * @return 表头信息
     */
    @Override
    public FirstArticleTitle getFaTitle(Long bentelerPlanId) {
        FirstArticleTitleBlob firstArticleTitleBlob = firstArticleTitleBlobService.getOne(new LambdaQueryWrapper<FirstArticleTitleBlob>()
                .eq(FirstArticleTitleBlob::getBentelerPlanId, bentelerPlanId));

        FirstArticleTitle firstArticleTitle = new FirstArticleTitle();
        try {
            firstArticleTitle = (FirstArticleTitle) this.byteArrayToObject(firstArticleTitleBlob.getTitleContent());
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return firstArticleTitle;
    }


    /**
     * 取得任务（首件、200件都是后台生成的任务，每个用户、计划、当天、类型（首件或200件））对应的应该是一个task。中断的task可以随便提交，所以要取状态是0初始化的，用户是-1这个userId不用传
     *
     * @param req
     * @return 任务
     */
    @Override
    public FirstArticleTask getFaTaskInfo(SearchFaTaskInfoReq req) {
        FirstArticleTask firstArticleTask = firstArticleMapper.getFaTaskInfo(req);
        return firstArticleTask;
    }

    /**
     * 查询中断最新的task
     *
     * @param req
     * @return 任务
     */
    @Override
    public FirstArticleTask getFaTaskInfoCatchLatest(SearchFaTaskInfoReq req) {
        FirstArticleTask firstArticleTask = firstArticleMapper.getFaTaskInfoCatchLatest(req);
        return firstArticleTask;
    }

    /**
     * 取得不放行任务对应的质量经理审批信息
     *
     * @param req
     * @return 不放行任务对应的质量经理审批信息
     */
    @Override
    public ResponseBean<FaNoGoTaskAapprovalRes> getFaNoGoTaskAapproval(SearchFaTaskReq req) {
        FaNoGoTaskAapprovalRes faNoGoTaskAapprovalRes = firstArticleMapper.getFaNoGoTaskAapproval(req);
        return ResponseBeanBuilder.ok(faNoGoTaskAapprovalRes);
    }

    /**
     * 取得所有计划
     *
     * @param req
     * @return 所有计划
     */
    @Override
    public ResponseBean<List<BentelerPlan>> getALLPlanList(SearchALLPlanListReq req) {
        List<BentelerPlan> aLLPlanList = planService.list(Wrappers.<BentelerPlan>lambdaQuery()
                .eq(BentelerPlan::getPlanType, req.getPlanType())
                .eq(BentelerPlan::getPauseFlag, false));
        return ResponseBeanBuilder.ok(aLLPlanList, Function.identity());
    }

    /**
     * 检查记录列表
     *
     * @param req
     * @return 检查记录列表
     */
    @Override
    public ResponseBean<FaCheckRecordListRes> getFaCheckRecordList(SearchFaCheckRecordListReq req) {
        FaCheckRecordListRes FaCheckRecordListRes = new FaCheckRecordListRes();
        List<FirstArticleWorkshop> firstArticleWorkshopList = firstArticleWorkshopService.list(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                .eq(FirstArticleWorkshop::getBentelerPlanId, req.getBentelerPlanId()));
        for (FirstArticleWorkshop firstArticleWorkshop : firstArticleWorkshopList) {
            List<FirstArticleItem> firstArticleItemList = firstArticleItemService.list(Wrappers.<FirstArticleItem>lambdaQuery()
                    .eq(FirstArticleItem::getWorkshopId, firstArticleWorkshop.getId()));
            for (FirstArticleItem firstArticleItem : firstArticleItemList) {
                /* 首件检查结果 */
                List<FirstArticleItemResult> firstItemResultList = this.getItemResultList(firstArticleItem.getId(), req.getCheckDate(), 0);
                firstArticleItem.setFirstItemResultList(firstItemResultList);

                /* 200件检查结果 */
                List<FirstArticleItemResult> twoHundredItemResultList = this.getItemResultList(firstArticleItem.getId(), req.getCheckDate(), 1);
                firstArticleItem.setTwoHundredItemResultList(twoHundredItemResultList);

                /* 中断检查结果(一天提交多个中断task，暂时都取出来了) */
                List<FirstArticleItemResult> catchItemResultList = this.getItemResultList(firstArticleItem.getId(), req.getCheckDate(), 2);
                firstArticleItem.setCatchItemResultList(catchItemResultList);

                /* 判断当前检查项目是否是放行 */
                SearchFaItemReleaseCountReq searchFaItemReleaseCountReq = new SearchFaItemReleaseCountReq();
                searchFaItemReleaseCountReq.setCheckDate(req.getCheckDate());
                searchFaItemReleaseCountReq.setFirstArticleItemId(firstArticleItem.getId());
                int count = firstArticleMapper.getFaItemReleaseCount(searchFaItemReleaseCountReq);
                if (count > 0) {
                    firstArticleItem.setReleaseFlag(1); // 1有
                } else {
                    firstArticleItem.setReleaseFlag(0); // 0没有
                }

            }

            firstArticleWorkshop.setFirstArticleItemList(firstArticleItemList);
        }
        FaCheckRecordListRes.setFirstArticleWorkshopList(firstArticleWorkshopList);

        return ResponseBeanBuilder.ok(FaCheckRecordListRes);
    }

    /**
     * 取得首件、200件、中断每天的点检记录
     *
     * @param req
     * @return 首件、200件、中断每天的点检记录
     */
    @Override
    public List<FirstArticleItemResult> getFaItemResultList(SearchFaItemResultListReq req) {
        List<FirstArticleItemResult> faItemResultList = firstArticleMapper.getFaItemResultList(req);
        return faItemResultList;
    }

    /**
     * 取得首件、200件、中断每天的点检记录
     *
     * @param firstArticleItemId
     * @param checkDate
     * @param typeFlag           0表示首件 1表示200件 2表示中断
     * @return 首件、200件、中断每天的点检记录
     */
    private List<FirstArticleItemResult> getItemResultList(long firstArticleItemId, String checkDate, int typeFlag) {
        SearchFaItemResultListReq searchFaItemResultListReq = new SearchFaItemResultListReq();
        searchFaItemResultListReq.setFirstArticleItemId(firstArticleItemId);
        searchFaItemResultListReq.setCheckDate(checkDate);
        searchFaItemResultListReq.setTypeFlag(typeFlag);

        List<FirstArticleItemResult> faItemResultList = this.getFaItemResultList(searchFaItemResultListReq);
        return faItemResultList;
    }

//    /**
//     * 判断检验结果是否填写完成
//     * @param req
//     * @return 检验结果是否填写完成
//     */
//	@Override
//	public FaAllCheckFlagRes getFaAllCheckFlag(SearchFaAllCheckFlagReq req) {
//		/* 取得task信息 */
//    	SearchFaTaskInfoReq searchFaTaskInfoReq = new SearchFaTaskInfoReq();
//    	searchFaTaskInfoReq.setBentelerPlanId(req.getBentelerPlanId());
//    	searchFaTaskInfoReq.setTypeFlag(req.getTypeFlag());
//    	
//    	FirstArticleTask faTaskInfo = null;
//    	if (req.getTypeFlag() == 0 || req.getTypeFlag() == 1) { // 0表示首件 1表示200件
//    		searchFaTaskInfoReq.setUserId(req.getUserId());
//    		faTaskInfo = this.getFaTaskInfo(searchFaTaskInfoReq);
//    	} else { // 2表示中断
//    		/* 取得中断最新的task */
//    		faTaskInfo = this.getFaTaskInfoCatchLatest(searchFaTaskInfoReq);
//    	}
//    	
//    	long firstArticleTaskId = 0;
//    	if (faTaskInfo != null) {
//    		firstArticleTaskId = faTaskInfo.getId();
//		}
//
//		SearchFaPlanReq searchFaPlanReq = new SearchFaPlanReq();
//		searchFaPlanReq.setBentelerPlanId(req.getBentelerPlanId());
//		int itemCount = firstArticleMapper.getFaItemCount(searchFaPlanReq);
//
//		SearchFaItemResultCountReq searchFaItemResultCountReq = new SearchFaItemResultCountReq();
//		searchFaItemResultCountReq.setBentelerPlanId(req.getBentelerPlanId());
//		searchFaItemResultCountReq.setFirstArticleTaskId(firstArticleTaskId);
//		int itemResultCount = firstArticleMapper.getFaItemResultCount(searchFaItemResultCountReq);
//
//		FaAllCheckFlagRes faAllCheckFlagRes = new FaAllCheckFlagRes();
//		if (itemCount == itemResultCount) {
//			faAllCheckFlagRes.setAllCheckFlag(1); // 全部完成
//		} else {
//			faAllCheckFlagRes.setAllCheckFlag(0); // 没完成
//		}
//
//		return faAllCheckFlagRes;
//	}


    /**
     * 取得用户列表
     *
     * @param req
     * @return 取得用户列表
     */
    public List<UserListRes> getUserList() {
        SearchUserListReq searchUserListReq = new SearchUserListReq();
        searchUserListReq.setRoleId(ROLE_ID);
        List<UserListRes> userList = firstArticleMapper.getUserList(searchUserListReq);
        return userList;
    }

    @Override
    public ResponseBean<FileRes> exportFaCheckRecordList(SearchFaCheckRecordListReq req) {
        BentelerPlan plan = planService.getById(req.getBentelerPlanId());

        List<FirstArticleWorkshop> firstArticleWorkshopList = getFaCheckRecordList(req).getData().getFirstArticleWorkshopList();
        FirstArticleTitle title = getFaTitle(req.getBentelerPlanId());
        Map<String, String> titleMap = Convert.toMap(String.class, String.class, title);
        titleMap.put("title", plan.getName());
        List<FaCalendarRes> faCalendarList = getFaCalendarList(SearchFaCalendarReq.builder().checkDate(req.getCheckDate()).build());
        titleMap.put("iniCheckData", faCalendarList.stream().filter(item2 -> item2.getTypeFlag().equals(0)).findFirst().map(item3 -> item3.getTaskTime().format(DateTimeFormatter.ofPattern("yyyy年MM月dd日"))).orElse("无"));
        titleMap.put("twoCheckData", faCalendarList.stream().filter(item2 -> item2.getTypeFlag().equals(1)).findFirst().map(item3 -> item3.getTaskTime().format(DateTimeFormatter.ofPattern("yyyy年MM月dd日"))).orElse("无"));
        titleMap.put("breakCheckData", faCalendarList.stream().filter(item2 -> item2.getTypeFlag().equals(2)).findFirst().map(item3 -> item3.getTaskTime().format(DateTimeFormatter.ofPattern("yyyy年MM月dd日"))).orElse("无"));
        titleMap.put("iniCheckPrinciple", faCalendarList.stream().filter(item2 -> item2.getTypeFlag().equals(0)).findFirst().map(item3 -> item3.getCheckPerson()).orElse("无"));
        titleMap.put("twoCheckPrinciple", faCalendarList.stream().filter(item2 -> item2.getTypeFlag().equals(1)).findFirst().map(item3 -> item3.getCheckPerson()).orElse("无"));
        titleMap.put("breakCheckPrinciple", faCalendarList.stream().filter(item2 -> item2.getTypeFlag().equals(2)).findFirst().map(item3 -> item3.getCheckPerson()).orElse("无"));


        ArrayList<Map<String, String>> arrayList = new ArrayList<>();
        //返回工作站结果
        firstArticleWorkshopList.forEach(item -> {
            Map<String, String> workMap = Convert.toMap(String.class, String.class, item);
            workMap.remove("firstArticleItemList");
            workMap.remove("allCheckFlag");
            workMap.remove("id");
            //返回巡检结果
            item.getFirstArticleItemList().forEach(item1 -> {

                Map<String, String> beanMap = Convert.toMap(String.class, String.class, item1);
                List<Integer> worksNumList = IntStream.rangeClosed(item1.getSnoStart(), item1.getSnoEnd()).boxed().collect(Collectors.toList());
                workMap.forEach((key, value) -> {
                    List<String> frequencyList = new ArrayList<String>();
                    beanMap.merge(key, value, (v1, v2) -> v1);
                    boolean numType = item1.getValFlag().equals(0);
                    String greadScope = numType ? item1.getSnoStart().toString() + "Nm - " + item1.getSnoEnd().toString() + "Nm" : (item1.getValOk().equals(0) ? "OK" : "NoK");
                    beanMap.put("valFlag", greadScope);

                    if (item1.getFirstFlag().equals(1)) {
                        frequencyList.add("首件");
                        Map<Integer, List<FirstArticleItemResult>> fisWorkMap = item1.getFirstItemResultList().stream().collect(Collectors.groupingBy(FirstArticleItemResult::getSno));
                        String greads = worksNumList.stream().map(work -> {
                            return fisWorkMap.get(work) != null ? fisWorkMap.get(work).stream().map(result -> {
                                if (numType) {
                                    return result.getVal();
                                } else {
                                    return result.getValDesc();
                                }
                            }).findFirst().orElse("未填写") : "未填写";
                        }).collect(Collectors.joining(","));
                        beanMap.put("inigread", greads);
                    } else {
                        beanMap.put("inigread", "------未要求检测------");
                    }
                    if (item1.getTwoHundredFlag().equals(1)) {
                        frequencyList.add("200件");
                        Map<Integer, List<FirstArticleItemResult>> fisWorkMap = item1.getTwoHundredItemResultList().stream().collect(Collectors.groupingBy(FirstArticleItemResult::getSno));
                        String greads = worksNumList.stream().map(work -> {
                            return fisWorkMap.get(work) != null ? fisWorkMap.get(work).stream().map(result -> {
                                if (numType) {
                                    return result.getVal();
                                } else {
                                    return result.getValDesc();
                                }
                            }).findFirst().orElse("未填写") : "未填写";
                        }).collect(Collectors.joining(","));
                        beanMap.put("twoGread", greads);
                    } else {
                        beanMap.put("twoGread", "------未要求检测------");
                    }
                    if (item1.getCatchFlag().equals(1)) {
                        frequencyList.add("中断");
                        Map<Integer, List<FirstArticleItemResult>> fisWorkMap = item1.getCatchItemResultList().stream().collect(Collectors.groupingBy(FirstArticleItemResult::getSno));
                        String greads = worksNumList.stream().map(work -> {
                            return fisWorkMap.get(work) != null ? fisWorkMap.get(work).stream().map(result -> {
                                if (numType) {
                                    return result.getVal();
                                } else {
                                    return result.getValDesc();
                                }
                            }).findFirst().orElse("未填写") : "未填写";
                        }).collect(Collectors.joining(","));
                        beanMap.put("breakOffgread", greads);
                    } else {
                        beanMap.put("breakOffgread", "------未要求检测------");
                    }
                    String collect = Stream.of(frequencyList.toArray(new String[frequencyList.size()])).collect(Collectors.joining("&"));
                    beanMap.put("frequency", collect);
                    beanMap.put("releaseRecord", item1.getReleaseFlag().equals(1) ? "存在" : "无");
                    beanMap.put("tool", item1.getToolId().equals(-1) ? "" : item1.getToolId().toString());
                    arrayList.add(beanMap);
                });
            });


        });

        // 模板注意 用{} 来表示你要用的变量 如果本来就有"{","}" 特殊字符 用"\{","\}"代替
        String templateFileName = "systemLib/template/首件计划表.xlsx";
        File tempFile = null;
        ExcelWriter excelWriter = null;
        try {
            // 模板文件
            tempFile = File.createTempFile("shoujian", ".xlsx");

            // 这里 会填充到第一个sheet， 然后文件流会自动关闭
            excelWriter = EasyExcel.write(tempFile).withTemplate(templateFileName).build();


            WriteSheet writeSheet = EasyExcel.writerSheet("首件")
                    .registerWriteHandler(new MyHandler()).build();
            //新一行
            FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.TRUE).build();

            excelWriter.fill(titleMap, writeSheet);
            excelWriter.fill(arrayList, fillConfig, writeSheet);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            excelWriter.finish();
        }
        return commonService.exportExcel(tempFile);
    }

    /**
     * 取得点检人
     *
     * @param req
     * @return 点检人
     */
    public List<FaCalendarRes> getFaCalendarList(SearchFaCalendarReq req) {
        List<FaCalendarRes> faCalendarList = firstArticleMapper.getFaCalendarList(req);
        return faCalendarList;
    }

    /**
     * Object转成byte[]
     *
     * @param obj
     * @return byte[]
     */
    private byte[] objectToByteArray(Object obj) throws Exception {
        byte[] bytes = null;
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ObjectOutputStream oos = new ObjectOutputStream(baos);
        oos.writeObject(obj);
        oos.close();
        bytes = baos.toByteArray();
        baos.close();
        return bytes;
    }

    /**
     * byte[]转成Object
     *
     * @param byte[]
     * @return Object
     */
    private Object byteArrayToObject(byte[] buffer) throws Exception {
        Object ob = null;
        ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(buffer));
        ob = ois.readObject();
        ois.close();
        return ob;
    }

}