package com.factory.web.service.impl.statistics;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.web.entity.fa.FirstArticleTask;
import com.factory.web.entity.fa.FirstArticleTaskHisBlob;
import com.factory.web.entity.pm.OnsiteTask;
import com.factory.web.entity.pm.OnsiteTaskHisBlob;
import com.factory.web.entity.statistics.ExceptionCountRes;
import com.factory.web.entity.statistics.StatisticsDateReq;
import com.factory.web.mapper.fa.FirstArticleTaskHisBlobMapper;
import com.factory.web.mapper.fa.FirstArticleTaskMapper;
import com.factory.web.mapper.pm.OnsiteTaskHisBlobMapper;
import com.factory.web.mapper.pm.OnsiteTaskMapper;
import com.factory.web.service.statistics.StatisticsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 异常统计
 *
 * @author sMArT
 * @version 1.0
 * @date 2021-07-22 4:03 下午
 */
@Service("statisticsService")
public class StatisticsServiceImpl implements StatisticsService {
    @Autowired
    private OnsiteTaskMapper onsiteTaskMapper;
    @Autowired
    private OnsiteTaskHisBlobMapper onsiteTaskHisBlobMapper;
    @Autowired
    private FirstArticleTaskMapper firstArticleTaskMapper;
    @Autowired
    private FirstArticleTaskHisBlobMapper firstArticleTaskHisBlobMapper;

    @Override
    public ResponseBean<ExceptionCountRes> getExceptionsByMonth(StatisticsDateReq statisticsDateReq) {

        String month = getMonth(statisticsDateReq);

        List<String> releasedDates = new ArrayList<>();
        List<String> recheckedDates = new ArrayList<>();

        //取首件放行状态数量
        List<FirstArticleTask> firstArticleTaskList = firstArticleTaskMapper.selectList(Wrappers.<FirstArticleTask>lambdaQuery()
                .eq(FirstArticleTask::getStatus, 2)
                .likeRight(FirstArticleTask::getTaskTime, month));
        firstArticleTaskList.forEach(firstArticleTask -> {
            DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            releasedDates.add(firstArticleTask.getTaskTime().format(fmt));
        });
        //去首件重检数量
        List<FirstArticleTaskHisBlob> firstArticleTaskHisBlobList = firstArticleTaskHisBlobMapper.selectList(Wrappers.<FirstArticleTaskHisBlob>lambdaQuery()
                .likeRight(FirstArticleTaskHisBlob::getCreatedTime, month));
        firstArticleTaskHisBlobList.forEach(firstArticleTaskHisBlob -> {
            DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            recheckedDates.add(firstArticleTaskHisBlob.getCreatedTime().format(fmt));
        });

        //取防错放行状态
        List<OnsiteTask> onsiteTaskList = onsiteTaskMapper.selectList(Wrappers.<OnsiteTask>lambdaQuery()
                .eq(OnsiteTask::getStatus, 2)
                .likeRight(OnsiteTask::getTaskTime, month));
        onsiteTaskList.forEach(onsiteTask -> {
            DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            releasedDates.add(onsiteTask.getTaskTime().format(fmt));
        });
        //取防错重检数量
        List<OnsiteTaskHisBlob> onsiteTaskHisBlobList = onsiteTaskHisBlobMapper.selectList(Wrappers.<OnsiteTaskHisBlob>lambdaQuery()
                .likeRight(OnsiteTaskHisBlob::getCreatedTime, month));
        onsiteTaskHisBlobList.forEach(onsiteTaskHisBlob -> {
            DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            recheckedDates.add(onsiteTaskHisBlob.getCreatedTime().format(fmt));
        });
        //合并去重
        List<String> exceptionDays = Stream.of(recheckedDates, releasedDates)
                .flatMap(Collection::stream).distinct().collect(Collectors.toList());

        return ResponseBeanBuilder.ok(ExceptionCountRes.builder()
                .exceptionCount(releasedDates.size() + recheckedDates.size())
                .recheckedCount(recheckedDates.size())
                .releasedCount(releasedDates.size())
                .exceptionDates(exceptionDays)
                .build());
    }

    /**
     * 验证日期格式
     *
     * @param statisticsDateReq 默认2021-06
     * @return
     */
    private String getMonth(StatisticsDateReq statisticsDateReq) {
        String month = "2021-06";
        try {
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM");
            Date date = simpleDateFormat.parse(statisticsDateReq.getMonth());
            month = simpleDateFormat.format(date);
        } catch (ParseException e) {
            e.printStackTrace();
            ResponseBeanBuilder.fail("年月格式错误");
        }
        return month;
    }
}
