package com.factory.wx.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.entity.po.remote.UserInfo;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.mapper.UserInfoMapper;
import com.factory.web.entity.fa.*;
import com.factory.web.entity.pm.*;
import com.factory.web.entity.pm.res.ApprovalUserRes;
import com.factory.web.mapper.BentelerPlanMapper;
import com.factory.web.mapper.fa.FirstArticleApprovalTaskMapper;
import com.factory.web.mapper.fa.FirstArticleCalendarMapper;
import com.factory.web.mapper.pm.ApprovalRuleMapper;
import com.factory.web.mapper.pm.OnsiteApprovalTaskMapper;
import com.factory.web.mapper.pm.OnsiteCalendarMapper;
import com.factory.web.mapper.pm.OnsiteTaskMapper;
import com.factory.web.service.BentelerPlanService;
import com.factory.web.service.fa.*;
import com.factory.web.service.pm.PreventMistakeService;
import com.factory.wx.entity.req.PlanApprovalTaskReq;
import com.factory.wx.entity.req.PlanInfoReq;
import com.factory.wx.entity.req.PlanPageReq;
import com.factory.wx.entity.res.PlanApprovalTaskRes;
import com.factory.wx.entity.res.PlanPassDetailRes;
import com.factory.wx.entity.res.PlanPassRes;
import com.factory.wx.mapper.PlanApprovalMapper;
import com.factory.wx.mapper.PlanPassMapper;
import com.factory.wx.service.PlanPassService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-07-13 12:42 下午
 */
@Service("planPassService")
public class PlanPassServiceImpl implements PlanPassService {

    private final static Logger LOGGER = LoggerFactory.getLogger(PlanPassServiceImpl.class);
    /**
     * 审批规则类型为：1新建计划审批
     */
    private static int APPROVAL_TYPE_NEW = 1;
    /**
     * 审批规则类型为：3防错
     */
    private static int APPROVAL_TYPE_PM = 3;
    @Autowired
    private PlanApprovalMapper planApprovalMapper;
    @Autowired
    private BentelerPlanMapper bentelerPlanMapper;
    @Autowired
    private OnsiteCalendarMapper onsiteCalendarMapper;
    @Autowired
    private ApprovalRuleMapper approvalRuleMapper;
    @Autowired
    private UserInfoMapper userInfoMapper;
    @Autowired
    private PreventMistakeService preventMistakeService;

    @Autowired
    private OnsiteApprovalTaskMapper onsiteApprovalTaskMapper;
    @Autowired
    private OnsiteTaskMapper onsiteTaskMapper;
    @Autowired
    private PlanPassMapper planPassMapper;
    @Autowired
    private FirstArticleCalendarMapper firstArticleCalendarMapper;
    @Autowired
    private FirstArticleApprovalTaskMapper firstArticleApprovalTaskMapper;
    @Autowired
    private BentelerPlanService planService;
    @Autowired
    private FirstArticleCalendarService firstArticleCalendarService;
    @Autowired
    private FirstArticleTitleBlobService firstArticleTitleBlobService;
    @Autowired
    private FirstArticleWorkshopService firstArticleWorkshopService;
    @Autowired
    private FirstArticleItemService firstArticleItemService;
    @Autowired
    private FirstArticleItemResultService firstArticleItemResultService;
    @Autowired
    private FirstArticleService firstArticleService;
    @Autowired
    private FirstArticleTaskService firstArticleTaskService;
//    @Autowired
//    private FirstArticleApprovalTaskMapper firstArticleApprovalTaskMapper;

    private static IPage<BentelerPlan> getBentelerPlanIPage(PlanPageReq planPageReq, List<OnsiteCalendar> onsiteCalendarList, BentelerPlanMapper bentelerPlanMapper) {
        IPage<BentelerPlan> bentelerPlanIPage;
        List<Long> planIds = onsiteCalendarList.stream().map(OnsiteCalendar::getBentelerPlanId).collect(Collectors.toList());

        LambdaQueryWrapper<BentelerPlan> wrapper = new LambdaQueryWrapper<>();
        wrapper.in(BentelerPlan::getId, planIds);
        wrapper.eq(BentelerPlan::getPlanType, planPageReq.getCheckType());

        if (StringUtils.isNotEmpty(planPageReq.getName())) {
            wrapper.like(BentelerPlan::getName, planPageReq.getName());
        }
        IPage<BentelerPlan> page = new Page<>(planPageReq.getPage(), planPageReq.getSize());
        bentelerPlanIPage = bentelerPlanMapper.selectPage(page, wrapper);
        return bentelerPlanIPage;
    }


    /**
     * 计划放行列表
     *
     * @param planPageReq
     * @return
     */
    @Override
    public PageBean<BentelerPlan> getPassListByPage(PlanPageReq planPageReq) {

        LambdaQueryWrapper<OnsiteApprovalTask> approvalTaskWrapper = new LambdaQueryWrapper<>();
        approvalTaskWrapper.eq(OnsiteApprovalTask::getApprovalUserId, planPageReq.getUserId());
        approvalTaskWrapper.eq(OnsiteApprovalTask::getStatus, 0);//获取未处理过的计划

        List<OnsiteApprovalTask> onsiteApprovalTaskList = onsiteApprovalTaskMapper.selectList(approvalTaskWrapper);
        IPage<BentelerPlan> bentelerPlanIPage = new Page<>();
        if (CollUtil.isNotEmpty(onsiteApprovalTaskList)) {
            List<Long> taskIds = onsiteApprovalTaskList.stream().map(OnsiteApprovalTask::getOnsiteTaskId).collect(Collectors.toList());
            List<OnsiteTask> onsiteTaskList = onsiteTaskMapper.selectBatchIds(taskIds);
            if (CollUtil.isNotEmpty(onsiteTaskList)) {
                List<Long> calendarIds = onsiteTaskList.stream().map(OnsiteTask::getOnsiteCalendarId).collect(Collectors.toList());
                List<OnsiteCalendar> onsiteCalendarList = onsiteCalendarMapper.selectBatchIds(calendarIds);
                if (CollUtil.isNotEmpty(onsiteCalendarList)) {
                    bentelerPlanIPage = getBentelerPlanIPage(planPageReq, onsiteCalendarList, bentelerPlanMapper);
                }
            }
        }
        return PageBean.<BentelerPlan>builder().row(bentelerPlanIPage.getRecords()).total(bentelerPlanIPage.getTotal()).page(bentelerPlanIPage.getPages()).build();
    }

    /**
     * 计划放行详情-首件
     *
     * @param planInfoReq
     * @return
     */

    @Override
    public PlanPassDetailRes getPassDetail(PlanInfoReq planInfoReq) {
        BentelerPlan bentelerPlan = bentelerPlanMapper.selectById(planInfoReq.getPlanId());

        List<FirstArticleCalendar> firstArticleCalendarList = firstArticleCalendarMapper.selectList(Wrappers.<FirstArticleCalendar>lambdaQuery()
                .eq(FirstArticleCalendar::getBentelerPlanId, planInfoReq.getPlanId()));
        List<String> usersName = new ArrayList<>();
        firstArticleCalendarList.forEach(onsiteCalendar -> {
            String name = assembleUserName(onsiteCalendar.getUserId(), onsiteCalendar.getShift());
            if (!name.isEmpty()) usersName.add(name);
        });

        //工作站
        List<FirstArticleWorkshop> firstArticleWorkshopList = firstArticleWorkshopService.list(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                .eq(FirstArticleWorkshop::getBentelerPlanId, planInfoReq.getPlanId()));
        // 每个workshop下的item
        for (int i = 0; i < firstArticleWorkshopList.size(); i++) {
            FirstArticleWorkshop workshop = firstArticleWorkshopList.get(i);
            List<FirstArticleItem> firstArticleItemList = firstArticleItemService.list(Wrappers.<FirstArticleItem>lambdaQuery()
                    .eq(FirstArticleItem::getWorkshopId, workshop.getId()));
            //每个item下的结果
            for (int j = 0; j < firstArticleItemList.size(); j++) {
                FirstArticleItem item = firstArticleItemList.get(j);
                List<FirstArticleItemResult> firstArticleItemResults = firstArticleItemResultService.list(Wrappers.<FirstArticleItemResult>lambdaQuery()
                        .eq(FirstArticleItemResult::getFirstArticleItemId, item.getId())
                        .eq(FirstArticleItemResult::getResult, 1));
                if (firstArticleItemResults.stream().count() == 0) {
                    firstArticleItemList.remove(j);
                } else {
                    // 首件列表
                    if (item.getFirstFlag() == 1 && firstArticleItemResults.stream().count() > 0) {
                        item.setFirstItemResultList(firstArticleItemResults);
                    }
                    // 200件
                    if (item.getTwoHundredFlag() == 1 && firstArticleItemResults.stream().count() > 0) {
                        item.setTwoHundredItemResultList(firstArticleItemResults);
                    }
                    // 中断
                    if (item.getCatchFlag() == 1 && firstArticleItemResults.stream().count() > 0) {
                        item.setCatchItemResultList(firstArticleItemResults);
                    }
                    if (firstArticleItemResults.stream().count() > 0) {
                        Optional<Long> taskId = firstArticleItemResults.stream().findFirst().map(FirstArticleItemResult::getFirstArticleTaskId);
                        if (taskId.isPresent()) {
                            FirstArticleTask firstArticleTask = firstArticleTaskService.getById(taskId.get().longValue());
                            item.setFirstArticleTask(firstArticleTask);
                            FirstArticleApprovalTask approvalTask = firstArticleApprovalTaskMapper.selectOne(Wrappers.<FirstArticleApprovalTask>lambdaQuery()
                                    .eq(FirstArticleApprovalTask::getFirstArticleTaskId, taskId.get().longValue()));
                            item.setFirstArticleApprovalTaskStatus(approvalTask.getStatus());
                        }
                    }
                }
            }
            if (firstArticleItemList.size() > 0) {
                workshop.setFirstArticleItemList(firstArticleItemList);
            } else {
                firstArticleWorkshopList.remove(i);
            }
        }


        //获取title
        FirstArticleTitle title = firstArticleService.getFaTitle(planInfoReq.getPlanId());

        return PlanPassDetailRes.builder()
                .planId(bentelerPlan.getId())
                .planName(bentelerPlan.getName())
                .effectiveDate(bentelerPlan.getEffectiveDate())
                .usersName(usersName)
                .firstArticleTitle(title)
                .firstArticleCalendars(firstArticleCalendarList)
                .firstArticleWorkshops(firstArticleWorkshopList)
                .build();
    }

    /**
     * 计划放行详情-防错
     *
     * @param planInfoReq
     * @return
     */
    @Override
    public PlanPassDetailRes getPassDetailPM(PlanInfoReq planInfoReq) {
        BentelerPlan bentelerPlan = bentelerPlanMapper.selectById(planInfoReq.getPlanId());
        List<OnsiteCalendar> onsiteCalendarList = onsiteCalendarMapper.selectList(Wrappers.<OnsiteCalendar>lambdaQuery()
                .eq(OnsiteCalendar::getBentelerPlanId, planInfoReq.getPlanId()));

        List<String> usersName = new ArrayList<>();
        onsiteCalendarList.forEach(onsiteCalendar -> {

            String name = assembleUserName(onsiteCalendar.getUserId(), null);
            if (!name.isEmpty()) usersName.add(name);
        });

        //获取title
        OnsiteTitle title = preventMistakeService.getTitle(planInfoReq.getPlanId());
        List<PlanApprovalTaskRes> planApprovalTaskResList = planPassMapper.getApprovalTaskList(planInfoReq.getUserId());

        PlanPassDetailRes planPassDetailRes = PlanPassDetailRes.builder()
                .planId(bentelerPlan.getId())
                .planName(bentelerPlan.getName())
                .effectiveDate(bentelerPlan.getEffectiveDate())
                .usersName(usersName)
                .titleInfo(title)
                .onsiteCalendars(onsiteCalendarList)
                .planApprovalTasks(planApprovalTaskResList)
                .effectiveDate(bentelerPlan.getEffectiveDate())
                .build();
        return planPassDetailRes;
    }

    /**
     * 计划放行
     *
     * @param planApprovalTaskReq
     * @return
     */
    @Override
    public ResponseBean approvalTaskOperate(PlanApprovalTaskReq planApprovalTaskReq) {

        int updateStatus = 0;
        if (planApprovalTaskReq.getPlanType() == 1) {
            FirstArticleApprovalTask ot = new FirstArticleApprovalTask();
            ot.setFirstArticleTaskId(planApprovalTaskReq.getTaskId());
            ot.setReason(planApprovalTaskReq.getReason());
            ot.setStatus(planApprovalTaskReq.getStatus());
            updateStatus = firstArticleApprovalTaskMapper.update(ot, Wrappers.<FirstArticleApprovalTask>lambdaQuery()
                    .eq(FirstArticleApprovalTask::getFirstArticleTaskId, planApprovalTaskReq.getTaskId())
                    .eq(FirstArticleApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()));
        }
        if (planApprovalTaskReq.getPlanType() == 2) {
            OnsiteApprovalTask ot = new OnsiteApprovalTask();
            ot.setOnsiteTaskId(planApprovalTaskReq.getTaskId());
            ot.setReason(planApprovalTaskReq.getReason());
            ot.setStatus(planApprovalTaskReq.getStatus());
            updateStatus = onsiteApprovalTaskMapper.update(ot, Wrappers.<OnsiteApprovalTask>lambdaQuery()
                    .eq(OnsiteApprovalTask::getOnsiteTaskId, planApprovalTaskReq.getTaskId())
                    .eq(OnsiteApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()));
        }

        if (updateStatus > 0) {
            //审批不通过触发下一级别
            if (planApprovalTaskReq.getStatus() == 2) {
                ApprovalRule approvalRuleMaxLevel = approvalRuleMapper.selectOne(Wrappers.<ApprovalRule>lambdaQuery()
                        .eq(ApprovalRule::getType, APPROVAL_TYPE_PM)
                        .orderByDesc(ApprovalRule::getApprovalLevel)
                        .last("limit 1"));
                //获取当前审批级别
                OnsiteApprovalTask onsiteApprovalTask = onsiteApprovalTaskMapper.selectOne(Wrappers.<OnsiteApprovalTask>lambdaQuery()
                        .eq(OnsiteApprovalTask::getOnsiteTaskId, planApprovalTaskReq.getTaskId())
                        .eq(OnsiteApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()).last("limit 1"));
                int userLevel = onsiteApprovalTask.getApprovalLevel();
                if (userLevel < approvalRuleMaxLevel.getApprovalLevel()) {
                    ApprovalUserRes approvalUserRes = preventMistakeService.getApprovalUserInfo(userLevel + 1, APPROVAL_TYPE_PM);
                    onsiteApprovalTaskMapper.insert(OnsiteApprovalTask.builder()
                            .onsiteTaskId(onsiteApprovalTask.getOnsiteTaskId())
                            .approvalLevel(approvalUserRes.getApprovalLevel())
                            .approvalUserId(approvalUserRes.getUserId())
                            .status(0)
                            .build());
                }
            }
            //审批通过
            if (planApprovalTaskReq.getStatus() == 1) {
                //审批通过，保存审批历史
                preventMistakeService.getApprovalUserInfo(1, APPROVAL_TYPE_PM);
            }
        }

        return ResponseBeanBuilder.ok();
    }

    /**
     * 计划放行列表
     *
     * @param planPageReq
     * @return
     */
    @Override
    public List<PlanPassRes> getPassList(PlanPageReq planPageReq) {
        List<PlanPassRes> planPassResList = new ArrayList<>();
        //首件列表
        if (planPageReq.getCheckType() == 1) {
            planPassResList = planPassMapper.getPlanPassList(planPageReq);
        }
        //防错列表
        if (planPageReq.getCheckType() == 2) {
            planPassResList = planPassMapper.getPlanPassPMList(planPageReq);
        }
        return planPassResList;
    }

    /**
     * byte[]转成Object
     *
     * @param buffer byte[]
     * @return Object
     */
    private Object byteArrayToObject(byte[] buffer) throws Exception {
        Object ob = null;
        ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(buffer));
        ob = ois.readObject();
        ois.close();
        return ob;
    }

    /**
     * 根据用户id及班次拼装用户姓名
     *
     * @param userId
     * @param shift
     * @return 取不到用户信息返回空字符串，取到用户信息拼接班次
     */
    private String assembleUserName(Long userId, String shift) {
        if (userId > 0) {
            UserInfo userInfo = userInfoMapper.selectById(userId);
            if (null != userInfo && null != userInfo.getName()) {
                return StringUtils.join(userInfo.getName(), " ", shift != null ? shift + "班" : "");
            }
        }
        return "";
    }

}
