package com.jihengbel.intelligent.applicationtask.init;

import com.jihengbel.intelligent.applicationtask.common.CommonUtil;
import com.jihengbel.intelligent.applicationtask.entry.OnsiteCalendar;
import com.jihengbel.intelligent.applicationtask.service.OnsiteTaskService;
import com.jihengbel.intelligent.applicationtask.websocket.WsSessionManager;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.web.socket.TextMessage;

import java.util.*;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.FutureTask;

/**
 * onsite 提醒job
 */
@DisallowConcurrentExecution
public class OnsiteRemindJob implements Job {


    private int remind_handle_flag=1;//设置提醒为已经处理
    private int type=2;//接错
    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        ConcurrentMap<String, Object> paramMap = (ConcurrentMap<String, Object>) jobExecutionContext.getJobDetail().getJobDataMap().get(QuartzManager.JOB_PARAM_NAME);
        try {
            createTask(paramMap);// 执行任务
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    /**
     * 创建Task主方法
     * @param paramMap
     */
    public void createTask(ConcurrentMap<String,Object> paramMap)  {
        Integer result=0;//没什么意义只是为了不出现整体并行
        List< FutureTask<Integer>> tasks=new ArrayList<>();
        tasks.add( getDayFutureTask(paramMap,true));
        tasks.add( getWeekFutureTask(paramMap,true));
        tasks.add( getMonthFutureTask(paramMap,true));
        tasks.add( getDayFutureTask(paramMap,false));
        tasks.add( getWeekFutureTask(paramMap,false));
        tasks.add( getMonthFutureTask(paramMap,false));
        try{
            for(FutureTask<Integer> t:tasks){
                t.get();
            }//等等所有线程结束
        }catch (Throwable e){
            e.printStackTrace();
        }

    }


    /**
     * day
     * @param paramMap
     * @return
     */
    private FutureTask<Integer> getDayFutureTask(ConcurrentMap<String, Object> paramMap,boolean isWork) {
        HashMap param = new HashMap();
        param.put("type_flag",0);
        Callable cal=null;
        if(isWork){
            param.put("remind_task_flag",0);
            cal=new remindOnsiteWork(paramMap,param);
        }else{
            param.put("remind_nowork_flag",0);
            cal=new remindOnsiteNoWork(paramMap,param);
        }
        FutureTask<Integer> dayTask = new FutureTask<Integer>(cal);
        Thread thread = new Thread(dayTask);
        thread.start();  //开启线程
        return dayTask;
    }

    /**
     * week
     * @param paramMap
     * @return
     */
    private FutureTask<Integer> getWeekFutureTask(ConcurrentMap<String, Object> paramMap,boolean isWork) {
        HashMap param = new HashMap();
        param.put("type_flag",1);
        Callable cal=null;
        if(isWork){
            param.put("remind_task_flag",0);
            cal=new remindOnsiteWork(paramMap,param);
        }else{
            param.put("remind_nowork_flag",0);
            cal=new remindOnsiteNoWork(paramMap,param);
        }
        FutureTask<Integer> weekTask = new FutureTask<Integer>(cal);
        Thread thread = new Thread(weekTask);
        thread.start();  //开启线程
        return weekTask;
    }

    /**
     * month
     * @param paramMap
     * @return
     */
    private FutureTask<Integer> getMonthFutureTask(ConcurrentMap<String, Object> paramMap,boolean isWork) {
        HashMap param = new HashMap();
        param.put("type_flag",2);
        Callable cal=null;
        if(isWork){
            param.put("remind_task_flag",0);
            cal=new remindOnsiteWork(paramMap,param);
        }else{
            param.put("remind_nowork_flag",0);
            cal=new remindOnsiteNoWork(paramMap,param);
        }
        FutureTask<Integer> monthTask = new FutureTask<Integer>(cal);
        Thread thread = new Thread(monthTask);
        thread.start();  //开启线程
        return monthTask;
    }



    /**
     * 发送提醒 需要工作的提醒
     */
    public class remindOnsiteWork implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;
        private HashMap paramQuery;


        public remindOnsiteWork(ConcurrentMap<String, Object> paramMap, HashMap paramQuery) {
            this.paramMap = paramMap;
            this.paramQuery = paramQuery;
        }




        @Override
        public Integer call() throws Exception {
            OnsiteTaskService onsiteTaskService = (OnsiteTaskService) paramMap.get("onsiteTaskService");
            List<OnsiteCalendar> cals = onsiteTaskService.getRemindWork(paramQuery);
            String day = null;
            String format = "yyyy-MM-dd HH:mm";
            for (OnsiteCalendar cal : cals) {
                if (cal.getStart_day() != null && cal.getStart_day().trim().length() > 0) {
                    day = cal.getTask_time() + " " + cal.getStart_day();
                    format = "yyyy-MM-dd HH:mm";
                } else {
                    day = cal.getTask_time();
                    format = "yyyy-MM-dd";
                }
                boolean flag = CommonUtil.getDateCompare(CommonUtil.getStrDate(day, format, 0), new Date(),false);
                if (flag) {
                    String token =String.valueOf(cal.getUser_id());
                    if(WsSessionManager.SESSION_POOL.containsKey(token)){
                        try {
                            WsSessionManager.SESSION_POOL.get(token).
                                    sendMessage(new TextMessage("有一个防错巡检需要处理，请及时操作！"));
                            //update  onsite_task set ${flag}=${flagVal} where id=${onsite_task_id}
                            Map<String,Object> qm=new HashMap<>();
                            qm.put("flag","remind_task_flag");
                            qm.put("flagVal",remind_handle_flag);
                            qm.put("onsite_task_id",cal.getOnsite_task_id());
                            onsiteTaskService.updateOnsiteRemindFlag(qm);
                        }catch (Throwable e){
                            e.printStackTrace();
                        }

                    }

                }

            }
            return 0;
        }
    }



    /**
     * 发送提醒  没有工作的提醒
     */
    public class remindOnsiteNoWork implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;
        private HashMap paramQuery;


        public remindOnsiteNoWork(ConcurrentMap<String, Object> paramMap, HashMap paramQuery) {
            this.paramMap = paramMap;
            this.paramQuery = paramQuery;
        }

        @Override
        public Integer call() throws Exception {
            OnsiteTaskService onsiteTaskService = (OnsiteTaskService) paramMap.get("onsiteTaskService");
            List<OnsiteCalendar> cals = onsiteTaskService.getRemindWork(paramQuery);
            String day = null;
            String format = "yyyy-MM-dd HH:mm";
            int min;
            for (OnsiteCalendar cal : cals) {
                if (cal.getStart_day() != null && cal.getStart_day().trim().length() > 0) {
                    day = cal.getTask_time() + " " + cal.getStart_day();
                    format = "yyyy-MM-dd HH:mm";
                    min=cal.getDay_outtime()+cal.getDay_validate();
                } else {
                    day = cal.getTask_time();
                    format = "yyyy-MM-dd";
                    min=0;
                }
                boolean flag = CommonUtil.getDateCompare(CommonUtil.getStrDate(day, format, min), new Date(),true);
                if (flag) {
                    Map map=new HashMap();
                    map.put("type",type);
                    Long  userid=onsiteTaskService.getRemindNoworkToManager(map);
                    if(userid!=null && userid>0){
                        map=new HashMap();
                        map.put("task_id",cal.getOnsite_task_id());
                        map.put("approval_user_id",userid);
                        map.put("task_type",type);
                        map.put("flag","remind_nowork_flag");
                        map.put("flagVal",remind_handle_flag);
                        map.put("onsite_task_id",cal.getOnsite_task_id());
                        onsiteTaskService.addRemindNowork(map);
//                        onsiteTaskService.updateOnsiteRemindFlag(map);

                        //需要推动调用这个 todo
                    }
                }

            }
            return 0;
        }
    }

}
